################################################################################
####################### ARCHAEOBOTANICAL ANALYSIS ##############################
################################################################################

############################## LIBRARIES #######################################

library(readxl)
library(dplyr)
library(tidyr)
library(expss)  
library(clam)

############################### SETTINGS #######################################

#### GENERAL ####

# DEFINE REGION OF ANALYSIS
#options: Europe, Iberia
region <- "Europe"

# DEFINE TIME WINDOW IN YEARS
tw_step = 100

# DEFINE PERIOD OF ANALYSIS IN YEARS BCE
maxAgeTimeFrame = -4000
minAgeTimeFrame = -500

# DEFINE IF ANALYSIS SHOULD BE DETRENDED
detrending <- FALSE
detr_mode <- "linear" #either "polynomial" or "linear"

#### ARCHAEOBOTANIC DATA ####

# DEFINE MINIMUM NUMBER OF GRAINS PER 100-yr TIMESLICE AND SITE
minCountsFilter = 20

# DEFINE CONSIDERED 14C DATA
#("no", 
#"site": all 14C used, 
#"structure": only structure and feature 14C used,
#"feature": only 14C from features used)
use14C = "site" 

# DEFINE CONFIDENCE RANGE OF 14C DATES
prob = 0.95 

# DEFINE IF 14C OUTLIERS SHOULD BE CONSIDERED
#FALSE: all 14C Age ranges should be used
#TRUE: only 14C dates within boundaries (+- tolerance) of archaeological dating range are used
excl14COutliers =TRUE 

# DEFINE TOLERANCE OF OUTLIERS IN YEARS
outlierTolerance = 100 

# DEFINE THRESHOLD FOR MASS FIND IN GRAINS PER 100 YEARS
massFind <- 1000

# DEFINE WHETHER FACTORS (AFTER EFFENBERGER) SHOULD BE USED FOR RI CALCULATION
applyFactors =TRUE

#### CLIMATIC DATA ####

# DEFINE FILTER FOR DATAPOINTS WITHIN TIME PERIOD FOR EACH ARCHIVE
minData <- 5

# DEFINE BANDWIDTH (SMOOTHING)
hw <- 200 

# DEFINE NUMBER OF REPLICATIONS
nreps <- 200

# DEFINE TIME STEP FOR INTERPOLATION
targstep <- 20

#################### FUNCTION FOR RI CALCULATION ###############################

### function to calculate the RI score values, twf is used to make RI calculation for different time window sizes comparable.
### Effenberger uses time slices of c. 500years, for this it would be twf =1, this factor is calculated later in the script
getRIScore <- function(grainCount, sumCount, relPerc, nrFeatures, twf, applyFactors){#twf= factor for relative changing of absolute count thresholds, depending on timewindow size
  #set criteria
  max_sum_all_grains = 1000/twf
  max_score_1 = 10/twf
  max_score_2 = 49/twf
  max_score_3 = 99/twf
  max_score_4 = 499/twf
  min_score_6 =25 #25%
  min_score_7 =50 #50%
  
  #set scores
  score_1 = 1 
  score_2 = 2
  score_3 = 3
  score_4 = 4
  score_5 = 5
  score_6 = 6
  score_7 = 7
  
  scores <- NULL
  
  for (i in 1:length(sumCount)){
    score = 0
    if(sumCount[i] > max_sum_all_grains)
    {
      if(relPerc[i] > min_score_7){score= score_7}
      else{
        if(relPerc[i] > min_score_6){score= score_6}
        else{
          if(grainCount[i]>max_score_4) {score= score_5}
        }
      }
    }
    
    if (score == 0){
      
      if(grainCount[i]>max_score_3){score=score_4}
      else{
        if(grainCount[i]>max_score_2){score= score_3}
        else{
          if(grainCount[i]>max_score_1){score=score_2}
          else {
            if(grainCount[i]>0){score=score_1}
          }
        }
      }
    }
    
    if(applyFactors){    
      #apply factor
      if(nrFeatures[i]>100){score = 4*score}
      else{
        if(nrFeatures[i]>40){score = 3*score}
        else{
          if(nrFeatures[i]>20){score = 2*score}
          else{
            if(sumCount[i]>10000/twf){score= 2*score}
          }
        }
      }
    }
    scores <- append(scores, score)
  }
  
  return(scores)
}


############################ LOAD DATA #########################################

if (region == "Europe") {
  path = "archaeobotanic data_NC_Europe_16_09_22.xlsx"
  data <- read_excel(path, sheet ="raw_data", na="NA") 
  data14C <- read_excel(path, sheet ="14C_dates", na="NA")} else if (region == "Iberia") {
  path = "archaeobotanic data_SE_Iberia_19_09_22.xlsx"
  data <- read_excel(path, sheet ="raw_data", na="NA") 
  data14C <- read_excel(path, sheet ="14C_dates", na="NA")} else {print("No or wrong region selected!")}

rm(path)

##################### DATE RANGE CALCULATION ###################################

#sum up counts and add column of total counts to data
data$total <- rowSums(data[,9:ncol(data)],na.rm=TRUE)

#delete entries/features with no taxa counts
data <- data %>% dplyr::filter(total > 0)

options(nwarnings = 10000)
if(use14C!="no"){
  
  #create empty fields
  data14C$max <- NA
  data14C$min <- NA
  
  #calculate age range for 14C dates and add min and max to data
  for(i in 1: nrow(data14C)){ 
    
    date1 <- hpd(caldist(data14C$date[i],data14C$error[i], cc=data14C$cc[i], BCAD=TRUE), prob=prob)
    data14C$min[i] <- date1[length(date1)/3*2]
    data14C$max[i] <- date1[1]
  }
  
  if(excl14COutliers){#check for and exclude 14C dates which are outside the indicated max/min ranges of given archaeological age estimates
    #get maximum archaeological age ranges for sites, structures and features
    dfArchAge_feature <- data %>% group_by(SiteID, FeatureID) %>% summarise(maxArchAge = min(start_date), minArchAge = max(end_date))
    
    dfArchAge_structure <- data %>% group_by(SiteID, StructureID) %>% summarise(maxArchAge = min(start_date), minArchAge = max(end_date))
    
    dfArchAge_site <- data %>% group_by(SiteID) %>% summarise(maxArchAge = min(start_date), minArchAge = max(end_date))
    
    #omit 14C  outside range
    for (i in 1: nrow(data14C)){
      #check if outside site range
      if (!is.na(data14C$FeatureID[i])){# 14C is from feature
        
        archAge <- dfArchAge_feature %>% filter(FeatureID==data14C$FeatureID[i] , SiteID==data14C$SiteID[i])
        
        if(length(archAge$maxArchAge)==0){
          warning("SiteID: ",data14C$SiteID[i],"and StructureeID: ",data14C$FeatureID[i]," in 14C table without corresponding entry in raw data")
          data14C$FeatureID[i] = NA
        }  
        else{
          
          if((data14C$max[i] <  archAge$maxArchAge - outlierTolerance  | data14C$min[i] > archAge$minArchAge + outlierTolerance  )){
            warning("14C age ",data14C$LabID[i],"\t from siteID: ", data14C$SiteID[i],"\t featureID: ",data14C$FeatureID[i], "\t out of date range and excluded!")
            data14C$FeatureID[i] = NA
            data14C$StructureID[i] = NA
            data14C$SiteID[i] = NA
          }
        }
      }
      else{
        if(!is.na(data14C$StructureID[i])){#14C is from structure
          
          archAge <- dfArchAge_structure %>% filter(StructureID == data14C$StructureID[i] , SiteID == data14C$SiteID[i])
          
          if(length(archAge$maxArchAge)==0){
            warning("SiteID: ",data14C$SiteID[i],"and StructureID: ",data14C$StructureID[i]," in 14C table without corresponding entry in raw data")
            data14C$StructureID[i] = NA
          }  
          else{
            if((data14C$max[i] <  archAge$maxArchAge - outlierTolerance  | data14C$min[i] > archAge$minArchAge + outlierTolerance  )){
              warning("14C age ",data14C$LabID[i],"\t from siteID: ", data14C$SiteID[i],"\t structureID: ",data14C$StructureID[i], "\t out of date range and excluded!")
              data14C$StructureID[i] = NA
              data14C$SiteID[i] = NA            
            }
          }
          
        }
        else{#14C is from site
          
          archAge <- dfArchAge_site %>% filter(SiteID == data14C$SiteID[i])
          
          if(length(archAge$maxArchAge)==0){
            warning("SiteID: ",data14C$SiteID[i]," in 14C table without corresponding entry in raw data")
            data14C$SiteID[i] = NA
          }
          else{
            if( data14C$max[i] <  archAge$maxArchAge - outlierTolerance  | data14C$min[i] > archAge$minArchAge + outlierTolerance  ){
              warning("14C age ",data14C$LabID[i],"\t from siteID: ", data14C$SiteID[i], " out of date range and excluded!")
              data14C$SiteID[i] = NA                 
            }
          }
        }
        
      }
    }#omit 14C
  }#exclude outliers
  
  
  df14C_feature <- data14C %>% group_by(SiteID, FeatureID)%>% summarise(maxAge14C = min(max), minAge14C =max(min))#group by SiteID and feature_ID and get min/max 14C Age range 
  df14C_feature <- df14C_feature %>% filter(!is.na(FeatureID))
  
  df14C_structure <- data14C %>% group_by(SiteID, StructureID)%>% summarise(maxAge14C = min(max), minAge14C =max(min))#
  df14C_structure <- df14C_structure %>% filter(!is.na(StructureID))
  
  df14C_site <- data14C %>% group_by(SiteID)%>% summarise(maxAge14C = min(max), minAge14C = max(min))#
  df14C_site <- df14C_site %>% filter(!is.na(SiteID))
  
  
  for (i in 1:nrow(data)){#Replace date ranges by 14C date ranges if 
    
    ###
    #to do: avoid NA in structure to calculate?
    ##
    
    range14CFeature <- df14C_feature %>% filter(SiteID == data$SiteID[i], FeatureID == data$FeatureID[i])
    range14CStructure <- df14C_structure %>% filter(SiteID == data$SiteID[i], StructureID == data$StructureID[i])
    range14CSite <- df14C_site %>% filter(SiteID == data$SiteID[i])
    
    if (length(range14CFeature$FeatureID)>0){ #check feature range
      
      if(data$start_date[i] - outlierTolerance < range14CFeature$maxAge14C  & data$end_date[i]  + outlierTolerance > range14CFeature$minAge14C#replace boundaries should be within tolerance
         & data$end_date[i]-data$start_date[i] > range14CFeature$minAge14C-range14CFeature$maxAge14C# replace range should be narrower
      ){
        
        warning("Age Range of feature ",data$FeatureID[i] ," from SiteID ",data$SiteID[i],": ",data$start_date[i], " to ", data$end_date[i], " replaced by: ", range14CFeature$maxAge14C, " to ", range14CFeature$minAge14C, " (based on 14C data from feature)")
        data$start_date[i] = range14CFeature$maxAge14C
        data$end_date[i] =  range14CFeature$minAge14C
      }
    }
    
    if ((use14C=="structure" | use14C =="site") & length(range14CStructure$StructureID) > 0){ #check structure range
      if(data$start_date[i]  - outlierTolerance < range14CStructure$maxAge14C & data$end_date[i] + outlierTolerance > range14CStructure$minAge14C 
         & data$end_date[i]-data$start_date[i] > range14CStructure$minAge14C-range14CStructure$maxAge14C
      ){
        
        warning("Age Range of feature ",data$FeatureID[i] ," from SiteID ",data$SiteID[i],": ",data$start_date[i], " to ", data$end_date[i], " replaced by: ", range14CStructure$maxAge14C, " to ", range14CStructure$minAge14C, " (based on 14C data from structure)")
        data$start_date[i] = range14CStructure$maxAge14C
        data$end_date[i] =  range14CStructure$minAge14C
      }
    }
    
    if (use14C=="site" & length(range14CSite$SiteID) > 0){ #check site range
      if(data$start_date[i]  - outlierTolerance < range14CSite$maxAge14C & data$end_date[i] + outlierTolerance > range14CSite$minAge14C 
         & data$end_date[i]-data$start_date[i] > range14CSite$minAge14C-range14CSite$maxAge14C
      ){
        
        warning("Age Range of feature ",data$FeatureID[i] ," from SiteID ",data$SiteID[i],": ",data$start_date[i], " to ", data$end_date[i], " replaced by: ", range14CSite$maxAge14C, " to ", range14CSite$minAge14C, " (based on 14C data from site)")
        data$start_date[i] = range14CSite$maxAge14C
        data$end_date[i] =  range14CSite$minAge14C
      }
    }   
    
  }
  
}#use14c

#display warnings
warnings()
warning("reset")
rm(date1,archAge,range14CFeature,range14CSite,range14CStructure,i,df14C_feature,
   df14C_site,df14C_structure,dfArchAge_feature,dfArchAge_site,dfArchAge_structure)

########### AORISTICS: DISTRIBUTE CEREAL COUNTS ACROSS TIMESLICES ##############

# replace all "NA" by "O"
df <- data %>% 
  dplyr::mutate_if(is.numeric, replace_na, replace = 0)

# define temporal bins
x_ts<- seq(maxAgeTimeFrame,minAgeTimeFrame, by=tw_step)
# get taxa names
taxa <- colnames(df)[9:ncol(df)] #taxa columns start at position 9

# prepare list for calculation
ts_data_Counts = vector(mode="list",length=length(taxa))
names(ts_data_Counts)<- taxa

for (i in 1:length(taxa)){
  
  taxa_counts <- data.frame(matrix(nrow=0, ncol=2+length(x_ts)))#
  colnames (taxa_counts)= c("SiteID","FeatureID",-x_ts)
  
  for(m in 1:nrow(df)){#iterate through features
    
    counts <- list(SiteID=df$SiteID[m],FeatureID=df$FeatureID[m])
    
    for (n in x_ts){#iterate through time slices
      {
        if(((n+tw_step/2) >= df$start_date[m]) & (n-tw_step/2)<= df$end_date[m]) {
          
          relCounts <-  df[[m,9+i-1]]/((df$end_date[m]-df$start_date[m])/tw_step) #calculate corrected/weighted counts for time slice
          
          if((n+tw_step/2)-df$start_date[m] < tw_step){ relCounts = relCounts * ((n+tw_step/2)-df$start_date[m])/tw_step}#if dating range only partially covers time window calculate only proportional counts
          if(df$end_date[m]-(n-tw_step/2) < tw_step){ relCounts = relCounts * (df$end_date[m]-(n-tw_step/2))/tw_step}
          
          counts[[length(counts)+1]] = relCounts
          
        } else {
          
          counts[[length(counts)+1]]=0
        }
        
        names(counts)[length(counts)]<- sprintf("X%s",-n) #name new list entry
      }
    }
    #add counts for one feature to taxa_counts
    taxa_counts <- rbind(taxa_counts,counts)
    
  }
  ts_data_Counts[[i]] <- taxa_counts
}

rm(counts,taxa_counts,n,m,i,relCounts,df)

ts_nrFeatures <- ts_data_Counts

for(n in 1:length(ts_data_Counts)){
  
  ts_nrFeatures[[n]][ts_nrFeatures[[n]] > 0] <- 1# replace all counts with "1" 
  ts_nrFeatures[[n]]$SiteID = ts_data_Counts[[n]]$SiteID #reset SiteID
  
  #delete redundant field FeatureID
  ts_nrFeatures[[n]] <- ts_nrFeatures[[n]] %>% select(-one_of("FeatureID"))
  ts_data_Counts[[n]] <- ts_data_Counts[[n]] %>% select(-one_of("FeatureID"))
  
  ts_nrFeatures[[n]] <- ts_nrFeatures[[n]] %>% group_by(SiteID) %>% summarise_if(is.numeric, sum)  
  ts_data_Counts[[n]] <- ts_data_Counts[[n]] %>% group_by(SiteID) %>% summarise_if(is.numeric, sum)
}

########################## FILTERING OF TIMESLICES #############################

# according to filter value 
if(minCountsFilter > 0){
  
  filterValue = minCountsFilter*tw_step/100
  
  for(n in 2: length(ts_data_Counts$total)){
    for(i in 1: nrow(ts_data_Counts$total)){
      
      if (ts_data_Counts$total[i,n] < filterValue){
        for (e in 1:length(ts_data_Counts)){
          ts_data_Counts[[e]][[i,n]] <- 0
        }
      }
    }
  } 
}
rm(filterValue)

############################## RI CALCULATION ##################################

ts_data_Scores <- ts_data_Counts
twf = 500/tw_step #factor for adapting thresholds for scoring to time window length, Effenberg time window c.500 yrs -> twf = 1

# RI is calculated per site and timeslice
for(i in 1:(length(ts_data_Counts)-1)){#run for all taxa excl. $total
  for(m in 2:(ncol(ts_data_Counts[[i]])))
  {ts_data_Scores[[i]][[m]] <- getRIScore(ts_data_Counts[[i]][[m]],#grainCounts
                                           ts_data_Counts[[length(ts_data_Counts)]][[m]],#totalCounts
                                           (ts_data_Counts[[i]][[m]])/(ts_data_Counts[[length(ts_data_Counts)]][[m]])*100 ,#relative percentages
                                           ts_nrFeatures[[i]][[m]], #nrFeatures
                                           twf, #factor
                                           applyFactors
    )
  }
}
rm(e,i,m,n,twf)

#################### CALCULATION OF RELATIVE PROPORTIONS #######################

ts_data_RelProp <- ts_data_Counts
for(i in 1:(length(ts_data_Counts)-1)){#run for all taxa excl. $total
   for(m in 2:(ncol(ts_data_Counts[[i]]))){
    ts_data_RelProp[[i]][[m]] <- (ts_data_Counts[[i]][[m]])/(ts_data_Counts[[length(ts_data_Counts)]][[m]])*100#relative percentages
    }
}
rm(i,m)

######################### DEFINITION OF MASS FINDS #############################

massFinds <- ts_data_Counts[["total"]]
massFinds[massFinds[] < massFind] <- 0 
massFinds[massFinds[] >= massFind] <- 1 
massFinds$SiteID = ts_data_Counts[["total"]]$SiteID #reset SiteID
massFinds <- massFinds %>%
  dplyr::select(-SiteID) %>%
  colSums()
nFeatures <- ts_nrFeatures[["total"]] %>%
  dplyr::select(-SiteID) %>%
  colSums()

####################### CALCULATION OF TIMESLICE MEANS #########################
ts_summed_Scores <- ts_data_Scores
ts_summed_Counts <- ts_data_Counts
ts_summed_relProp <- ts_data_RelProp

ts_nrsites <- count_col_if(gt(0), ts_data_Counts$total[2:(ncol(ts_data_Counts$total))])
for(n in 1:length(ts_summed_Scores)){
  
  ts_summed_Scores[[n]] <-  ts_summed_Scores[[n]] %>% select(-one_of("SiteID"))
  ts_summed_Scores[[n]] <- ts_summed_Scores[[n]] %>%  summarise_if(is.numeric, sum)
  
  ts_summed_Counts[[n]] <-  ts_summed_Counts[[n]] %>% select(-one_of("SiteID"))
  ts_summed_Counts[[n]] <- ts_summed_Counts[[n]] %>%  summarise_if(is.numeric, sum)
  
  ts_summed_relProp[[n]] <- ts_summed_relProp[[n]] %>% select(-one_of("SiteID"))
  ts_summed_relProp[[n]] <- ts_summed_relProp[[n]] %>% summarise(across(where(is.numeric), ~ mean(.x, na.rm = TRUE))) 
  
}
rm(n,ts_nrFeatures)

############################### CREATE OUTPUT ##################################

counts <- data.frame()
for(n in 1:(length(ts_summed_Counts)-1)){# -1 to exclude df total
  df <- data.frame(score = t(ts_summed_Counts[[n]]))
  df$taxa <- names(ts_summed_Counts[n])
  df$age <- x_ts
  df$var <- "counts"
  counts <- rbind(counts,df)
}

RI <- data.frame()
for(n in 1:(length(ts_summed_Scores)-1)){# -1 to exclude df total
  df <- data.frame(score = t(ts_summed_Scores[[n]]))
  df$taxa <- names(ts_summed_Scores[n])
  df$age <- x_ts
  df$var <- "RI"
  RI <- rbind(RI,df)
}

prop <- data.frame()
for(n in 1:(length(ts_summed_relProp)-1)){# -1 to exclude df total
  df <- data.frame(score = t(ts_summed_relProp[[n]]))
  df$taxa <- names(ts_summed_relProp[n])
  df$age <- x_ts
  df$var <- "proportion"
  prop <- rbind(prop,df)
} 

nsites <- data.frame(score = ts_nrsites,
                     taxa = "amount",
                     age = x_ts,
                     var = "sites")
nMassFinds <- data.frame(score = massFinds,
                         taxa = "amount",
                         age = x_ts,
                         var = "mass finds")
nfeatures <- data.frame(score = nFeatures,
                        taxa = "amount",
                        age = x_ts,
                        var = "features")
propmassfinds <- data.frame(score = massFinds/nFeatures*100,
                            taxa = "proportion",
                            age = x_ts,
                            var = "rel mass finds")

archbot_final <- bind_rows(counts,RI,prop,nsites,nMassFinds,nfeatures,propmassfinds)

# EXPORT
path <- paste0("../data/archaeobotanic_aoristic_data_",region,"_",maxAgeTimeFrame,"_to_",
               minAgeTimeFrame,"_by_",tw_step,"yrs_filter_",minCountsFilter,"_massfind_",massFind,".csv")
archbot_export <- archbot_final %>%
  tidyr::pivot_wider(names_from = c(var,taxa),
                     values_from = score)

write.csv(x = archbot_export,
           file = path,
           row.names = FALSE)

rm(df,counts,RI,prop,n,ts_nrsites,massFinds,ts_data_Counts,ts_data_RelProp,
   ts_data_Scores,ts_summed_Counts,ts_summed_Scores,nsites,archbot_export,
   nMassFinds,nFeatures,nfeatures,propmassfinds,taxa,path)

################################################################################
####################### PALAEOCLIMATOLOGICAL ANALYSIS ##########################
################################################################################

################################ LIBRARIES #####################################

library(locfit)

############################### LOAD DATA ######################################

if (region == "Europe") {
  archives <- read.csv("NEurope_clim_archives.csv") } else if (region == "Iberia") {
      archives <- read.csv("SEIberia_clim_archives.csv") }

temp <- list.files(".", "+[0-9]_raw_Bacon", recursive=T, all.files=T)
files <- lapply(temp, read.csv) #load all raw data into a single list
#convert BCE ages to BP
minBP <- minAgeTimeFrame * -1 + 1950
maxBP <- maxAgeTimeFrame * -1 + 1950

raw_data <- do.call(rbind.data.frame,files) %>% #transform list into a single dataframe
  dplyr::filter(ID %in% archives$ID,
                !is.na(data),
                age >= minBP - 500, # filter ages with 500 yr egde size
                age <= maxBP + 500)

rm(path,files,temp,minBP,maxBP)

############################ DOWNSCALING #######################################

temp_res <- raw_data %>%
  dplyr::group_by(ID) %>%
  dplyr::mutate(min_age = min(age),
                max_age = max(age),
                n_data = length(data),
                resolution = (max_age - min_age)/n_data) %>%
  dplyr::ungroup() %>%
  distinct(ID, .keep_all = TRUE) %>%
  dplyr::mutate(mean_res = mean(resolution))

ds_archives <- temp_res %>%
  dplyr::filter(resolution < 25,
                n_data > 1)
ds_data <- raw_data %>%
  dplyr::filter(ID %in% ds_archives$ID)
ds_IDs <- ds_archives %>%
  dplyr::pull(ID)

bin_list <- list()
for (j in 1:length(ds_archives$ID)) {
  
  sub_data <- ds_data %>%
    dplyr::filter(ID == ds_IDs[j])
  bins <- seq(min(sub_data$age),
              max(sub_data$age), 
              by = 25)
  sub_list <- list()
  
  for(i in 1:(length(bins)-1)) {
    mean_age <- data.frame(ID = ds_IDs[j], mean_age = bins[i]+12.5)
    binned_data <- sub_data %>%
      dplyr::filter(age >= bins[i] & age <= bins[i+1]) %>%
      dplyr::mutate(mean(data)) %>%
      dplyr::distinct(ID, .keep_all = TRUE) %>%
      dplyr::left_join(.,mean_age, by= "ID") %>%
      dplyr::select(ID,mean_age,"mean(data)") %>%
      dplyr::rename("age" = "mean_age") %>%
      dplyr::rename("data" = "mean(data)")
    
    if(length(binned_data$ID) == 0){
      binned_data <- dplyr::bind_rows(binned_data, c(ID=NA,
                                                     age=NA,
                                                     data=NA))
    } else {binned_data <- binned_data
    }
    
    sub_list[[i]] <- binned_data
    
    rm(mean_age, binned_data)  
  }
  ds_subdata <- data.frame(matrix(unlist(sub_list), nrow = length(sub_list), byrow = T)) %>%
    dplyr::rename("ID" = "X1") %>%
    dplyr::rename("age" = "X2") %>%
    dplyr::rename("data" = "X3")
  bin_list[[j]] <- ds_subdata
  rm(sub_list,ds_subdata,bins)
}

clim_final <- raw_data %>%
  dplyr::filter(!(ID %in% ds_IDs)) %>%
  dplyr::select(-depth_original)
for (n in 1:length(ds_IDs)) {
  clim_final <- clim_final %>%
    dplyr::bind_rows(., bin_list[[n]])
}

clim_final <- clim_final %>%
  dplyr::filter(!is.na(data))

rm(bin_list,ds_archives,ds_data,temp_res,ds_IDs,i,j,n,sub_data)

######################### DATA PREPARATION #####################################

# NORMALIZATION
n_data <- clim_final %>%
  dplyr::left_join(.,archives, by = "ID") %>%
  dplyr::mutate(
    n_data = case_when(
      Normalization_needed == "Y" ~ data * -1,
      Normalization_needed == "N" ~ data)) %>%
  dplyr::select(ID,age,n_data)

# Z-SCORE CALCULATION
z_data <- n_data %>%
  dplyr::group_by(ID) %>%
  dplyr::filter(n() >= minData) %>%
  dplyr::mutate(z_score = 
                  (n_data - mean(n_data, na.rm = T)) / 
                  sd(n_data, na.rm = T)) %>%
  dplyr::select(ID,age,z_score)
rm(n_data,clim_final)

if (region == "Europe") {
  z_data <- z_data %>%
    dplyr::mutate(reg.ID = "Europe")} else if (region == "Iberia") {
      z_data <- z_data %>%
        dplyr::mutate(reg.ID = "Iberia")
    } 

z_data <- z_data %>%
  dplyr::left_join(archives, by = "ID") %>%
  dplyr::mutate(Season = "combined",
                age = (age - 1950)*-1) %>%
  dplyr::select(ID,SiteID,reg.ID,Parameter,Season,age,z_score)

# DETRENDING
if(detrending == TRUE) {
  if (detr_mode == "polynomial") {
  d_model <- z_data %>%                                           
    dplyr::group_by(ID) %>%
    dplyr::filter(!is.na(z_score)) %>%
    dplyr::mutate(smooth = predict(
      loess(z_score ~ age, span = 0.9, 
            familiy = "symmetric", degree = 2)))} else if (detr_mode == "linear"){
              d_model <- z_data %>%                                           
              dplyr::group_by(ID) %>%
              dplyr::filter(!is.na(z_score)) %>%
              dplyr::mutate(smooth = predict(
                lm(z_score ~ age)))} else {
                  print("Wrong detrending mode selected!")}
  z_data <- d_model %>%
    dplyr::mutate(z_score = z_score - smooth) %>%
    dplyr::select(-smooth)} else {
      z_data <- z_data}
rm(d_model)

# DEFINE DESIRED TIME INTERVAL
targbeg <- minAgeTimeFrame
targend <- maxAgeTimeFrame

# DEFINE VARIABLES AND SEASONS
vars <- z_data %>% dplyr::pull(Parameter) %>% unique()
# var <- vars[1]

timeseries <- sapply(X = vars,
                     simplify = FALSE,
                     USE.NAMES = TRUE,
                     FUN = function(var) {
                       sub_data <- z_data[z_data$Parameter == var,]
                       sites <- sub_data %>%
                         dplyr::select(ID,SiteID) %>%
                         distinct()
                 if(nrow(sub_data) == 0) {return()}
                 
                 maxrecs <- max(sub_data %>%
                                  dplyr::group_by(ID) %>%
                                  dplyr::group_size(.))
                 maxreps <- length(unique(sub_data$ID))
                 age <- matrix(NA, ncol=maxreps, nrow=maxrecs)
                 z_score <- matrix(NA, ncol=maxreps, nrow=maxrecs)
                 targage <- data.frame(x=seq(targbeg,targend,-targstep))
                 yfit <- matrix(NA, nrow=length(targage$x), ncol=nreps)
                 
                 ii <- 0
                 for (i in 1:maxreps) {
                   
                   sitenum <- sites[i,1]
                   siteidchar <- as.character(sitenum)
                   
                   indata <- z_data %>%
                     dplyr::filter(ID == siteidchar)
                   nsamp <-  length(indata$age)
                   
                   if (nsamp > 0) {
                     ii <- ii+1
                     age[1:nsamp,ii] <- indata$age 
                     z_score[1:nsamp,ii] <- indata$z_score
                     
                   }
                   rm(sitenum,siteidchar,indata,nsamp)
                 }
                 nsites <- ii
                 
                 x <- as.vector(age)
                 y <- as.vector(z_score)
                 lfdata <- data.frame(x,y)
                 lfdata <- na.omit(lfdata)
                 x <- lfdata$x; y <- lfdata$y
                 
                 # 2. locfit
                 # initial fit, unresampled (i.e. all) data
                 loc01 <- locfit(y ~ lp(x, deg=2, h=hw), maxk=100, family="gaussian") #maxk = 100 is fine. greater gives greater uncertainty, use familiy = gaussian
                 
                 pred01 <- predict(loc01, newdata=targage, se.fit=TRUE)
                 loc01_fit <- data.frame(targage$x, pred01$fit)
                 fitname <- paste("locfit_",as.character(hw), sep="")
                 colnames(loc01_fit) <- c("age", fitname)
                 rm(x,y,lfdata)
                 
                 j <- 100
                 for (j in 1:nreps) {
                   
                   randsitenum <- sample(seq(1:nsites), nsites, replace=TRUE)
                   
                   x <- as.vector(age[,randsitenum])
                   y <- as.vector(z_score[,randsitenum])
                   lfdata <- data.frame(x,y)
                   lfdata <- na.omit(lfdata)
                   x <- lfdata$x; y <- lfdata$y
                   locboot <- locfit(y ~ lp(x, deg=2, h=hw), maxk=100, maxit=20, family="gaussian")
                   predboot <- predict(locboot, newdata=targage, se.fit=TRUE)
                   yfit[,j] <- predboot$fit
                   
                   rm(randsitenum,x,y,lfdata)
                 }
                 
                 fitname <- paste("locfit_",as.character(hw), sep="")
                 colnames(loc01_fit) <- c("age", fitname)
                 
                 yfit95 <- apply(yfit, 1, function(x) quantile(x,prob=0.975, na.rm=T))
                 yfit05 <- apply(yfit, 1, function(x) quantile(x,prob=0.025, na.rm=T))
                 
                 final <- loc01_fit %>%
                   bind_cols("lower" = yfit05,
                             "upper" = yfit95)
                 })

prec_data <- timeseries[["prec"]] %>%
  dplyr::mutate(var = "prec")
temp_data <- timeseries[["temp"]] %>%
  dplyr::mutate(var = "temp")
clim_data <- bind_rows(prec_data,temp_data)

# EXPORT
if(detrending == TRUE) {
  if (detr_mode == "polynomial") {
    text1 <- "detrended_polynomial"
  } else {
    text1 <- "detrended_linear"
  }
} else { text1 <- "undetrended"}
path <- paste0("../data/composite_climate_data_",targend,"_",targbeg,
               "_",region,"_seasons_combined_",text1,".csv")
write.csv2(x = clim_data,
           file = path)
rm(timeseries,prec_data,temp_data,vars,path,archives)

################################################################################
############################# CORRELATION TESTS ################################
################################################################################

################################ LIBRARIES #####################################

library(Hmisc)

######################### BINNING OF CLIMATE DATA ##############################

bins <- data.frame(min = seq(minAgeTimeFrame+50,maxAgeTimeFrame+50,-100),
                   max = seq(minAgeTimeFrame-50,maxAgeTimeFrame-50,-100)) %>%
  tidyr::unite("bin",min:max,sep = "-", remove = F)

clim_data2 <- clim_data %>%
  tidyr::pivot_wider(names_from = "var",
                     values_from = c("locfit_200","lower","upper")) %>%
  dplyr::rename(mean_prec = locfit_200_prec,
                mean_temp = locfit_200_temp) %>%
  dplyr::select(age,mean_prec,mean_temp)

clim_bin <- clim_data2 %>%
  cut(x = .$age, 
      breaks = seq(minAgeTimeFrame+50,maxAgeTimeFrame-50,-100),
      labels = bins$bin,
      include.lowest = T, 
      right = T) %>%
  dplyr::bind_cols(clim_data2) %>%
  dplyr::rename("bins" = "...1") %>%
  dplyr::filter(!is.na(bins)) %>%
  dplyr::select(bins,
                contains("mean")) %>%
  dplyr::group_by(bins) %>%
  dplyr::mutate(across(.cols = mean_prec:mean_temp, ~ mean(.x, na.rm = TRUE))) %>%
  distinct() %>%
  dplyr::bind_cols(age = seq(maxAgeTimeFrame,minAgeTimeFrame,100))
rm(clim_data2)

#################### DETRENDING OF CEREAL PROPORTIONS ##########################

if(detrending == TRUE) {
  if (detr_mode == "polynomial") {
    d_model <- archbot_final %>% 
      dplyr::filter(var == "proportion") %>%
      dplyr::group_by(taxa) %>%
      dplyr::mutate(smooth = predict(
        loess(score ~ age, span = 0.9, 
              familiy = "symmetric", degree = 2)))} else if (detr_mode == "linear"){
                d_model <- archbot_final %>% 
                  dplyr::filter(var == "proportion") %>%
                  dplyr::group_by(taxa) %>%
                  dplyr::mutate(smooth = predict(
                    lm(score ~ age)))} else {
                      print("Wrong detrending mode selected!")}
  archbot_prop <- d_model %>%
    dplyr::mutate(score = score - smooth) %>%
    dplyr::select(-smooth)
  rm(d_model)} else {
      archbot_prop <- archbot_final %>% 
        dplyr::filter(var == "proportion")}

#calculate percentages
archbot_prop <- archbot_prop %>%
  dplyr::group_by(age) %>%
  dplyr::mutate(total = sum(score),
                proportion = score/total*100) %>%
  dplyr::ungroup()

########################## SPEARMAN CORRELATION ################################

corrdf <- archbot_prop %>%
  dplyr::select(-var,-total,-score) %>%
  tidyr::pivot_wider(names_from = taxa,
                     values_from = proportion) %>%
  dplyr::left_join(clim_bin, by = "age") %>%
  dplyr::select(-bins,-age)

if (region == "Europe") {
  corrdf <- corrdf %>%
    dplyr::mutate(barley_all = `naked barley` + `hulled barley`,
                  wheat_all = `emmer` + `einkorn` + `spelt` + `free-threshing wheat`)
} else if (region == "Iberia") {
  corrdf <- corrdf %>%
    dplyr::mutate(barley_all = `naked barley` + `hulled barley`,
                  wheat_all = `emmer` + `einkorn` + `free-threshing wheat`)
} else {print("Wrong or no region selected!")}

corr <- corrdf %>%
  as.matrix() %>%
  rcorr(type = "spearman")

#prepare output table with only highly correlated and significant data
rho <- corr[["r"]] %>%
  as.data.frame() %>%
  dplyr::mutate(var1 = row.names(.)) %>%
  pivot_longer(!var1,
               names_to = "var2",
               values_to = "rho") 
p <- corr[["P"]] %>%
  as.data.frame() %>%
  dplyr::mutate(var1 = row.names(.)) %>%
  pivot_longer(!var1,
               names_to = "var2",
               values_to = "p")

corr_data <- rho %>%
  dplyr::bind_cols(p$p) %>%
  dplyr::rename(p = "...4") %>%
  dplyr::filter(!is.na(p),
                p <= .005) %>%
  distinct(rho, .keep_all = TRUE)

path <- paste0("../data/significant_correlations_from_proportions_",region,"_",maxAgeTimeFrame,"_to_",
               minAgeTimeFrame,"_bins_",tw_step,"_filter_",minCountsFilter,"_",text1,".csv")
write.csv2(x = corr_data,
          file = path,
          row.names = F)
rm(path,corr,corrdf,bins,p,rho,corr_data)

################################################################################
############################## CONISS ANALYSIS #################################
################################################################################

################################ LIBRARIES #####################################

library(rioja)

############################### CALCULATION ####################################

#create matrix from relative proportion data
mt <-matrix(unlist(ts_summed_relProp[-length(ts_summed_relProp)]), 
            nrow= length(ts_summed_relProp[[1]]))
rownames(mt) <- x_ts

#calculate relative percentages per time slice
mt <- round(mt/rowSums(mt)*100,2)

#remove time slice with no data
mt <- mt[!is.nan(mt[,1]),]

#create dissimilarity matrix and run constrained clustering
archbot_diss <- dist(sqrt(mt))
archbot_clust <- chclust(archbot_diss)

#plot results
path <- paste0("../plots/CONISS_cereal_proportions_",region,"_filter_",minCountsFilter,".png")
png(path,
    width = 560,
    height = 560,
    res = 72)
par(mfcol = c(2,1),
    mar = c(4,4,1,2))
plot(archbot_clust, 
     hang=-1, 
     main=paste("CONISS","Cereal data",region, sep=" ")
)
bstick(archbot_clust)
dev.off()

## CONISS Climate data##
mt_clim <- clim_bin %>% 
  dplyr::ungroup() %>% 
  select(mean_prec, mean_temp) %>%
  as.matrix()
rownames(mt_clim) <- clim_bin$age

clim_diss <- dist(mt_clim)
clim_clust <- chclust(clim_diss)

path <- paste0("../plots/CONISS_clim_data_",region,"_",text1,".png")
png(path)
plot(clim_clust, 
     hang=-1, 
     main=paste("CONISS","Climate data",region, sep=" "))
dev.off()

rm(clim_bin,clim_clust,mt,mt_clim,archbot_diss,clim_diss,path,x_ts)
 
################################################################################
################################# PLOTTING #####################################
################################################################################

################################ LIBRARIES #####################################

library(ggplot2)
library(facetscales)

################################ SETTINGS ######################################

# SET Y-AXIS LIMITS AND COLOURS
if(region == "Iberia") {
  taxalist <- c("millet","naked barley","hulled barley","free-threshing wheat","einkorn","emmer")
  taxaColors <- c("yellow","darkolivegreen1","darkolivegreen3","darkorange","darkorange3","darkorange4")
  prec_label_y <- .5
  temp_label_y <- 2.5
  scales_y <- list(
    `prec` = scale_y_continuous(limits = c(-.75, .75), breaks = seq(-1, 1, .5)),
    `temp` = scale_y_continuous(limits = c(-3, 3), breaks = seq(-2, 2, 1)),
    `sites` = scale_y_continuous(limits = c(0, 15), breaks = seq(0, 20, 5)),
    `features` = scale_y_continuous(limits = c(0, 200), breaks = seq(0, 200, 50)),
    `mass finds` = scale_y_continuous(limits = c(0, 10), breaks = seq(0, 10, 5)),
    `rel mass finds` = scale_y_continuous(limits = c(0,20), breaks = seq(0,20,5)),
    `proportion` = scale_y_continuous(limits = c(0, 1), breaks = seq(0, 1, .25)),
    `RI` = scale_y_continuous(limits = c(0, 1), breaks = seq(0, 1, .25)),
    `counts` = scale_y_continuous(limits = c(0, 175000), breaks = seq(0, 200000, 50000))
  )
} else if (region == "Europe") {
  taxalist <- c("broomcorn millet","naked barley","hulled barley","spelt","free-threshing wheat","einkorn","emmer")
  taxaColors <- c("yellow","darkolivegreen1","darkolivegreen3","orange","darkorange","darkorange3","darkorange4")
  prec_label_y <- 1.2
  temp_label_y <- .4
  scales_y <- list(
    `prec` = scale_y_continuous(limits = c(-1, 2), breaks = seq(-1, 2, 1)),
    `temp` = scale_y_continuous(limits = c(-1.2, .6), breaks = seq(-1, 0, 1)),
    `sites` = scale_y_continuous(limits = c(0,25), breaks = seq(0,30,10)),
    `features` = scale_y_continuous(limits = c(0, 700), breaks = seq(0, 600, 200)),
    `mass finds` = scale_y_continuous(limits = c(0,10), breaks = seq(0,10,5)),
    `rel mass finds` = scale_y_continuous(limits = c(0,5), breaks = seq(0,5,1)),
    `proportion` = scale_y_continuous(limits = c(0,1), breaks = seq(0,1,.25)),
    `RI` = scale_y_continuous(limits = c(0,1), breaks = seq(0,1,.25)),
    `counts` = scale_y_continuous(limits = c(0,150000), breaks = seq(0,150000,50000))
  )
} else {print("No or wrong region selected!")}

prec_label <- z_data %>% 
  dplyr::ungroup() %>%
  dplyr::filter(Parameter == "prec") %>%
  distinct(ID) %>%
  nrow()
temp_label <- z_data %>% 
  dplyr::ungroup() %>%
  dplyr::filter(Parameter == "temp") %>%
  distinct(ID) %>%
  nrow()

############## PLOT: DATA WITH MASS FIND AND FEATURE COUNTS ####################

layout <- ggplot(archbot_final %>%
                   dplyr::filter(var %in% c("proportion","RI","counts","mass finds","features","sites")), 
                 aes(age, fill = factor(taxa, levels = taxalist))) +
  facet_grid_sc(rows = vars(factor(var,
                                   levels = c("proportion","RI","counts","mass finds","features","sites"))),
                scales = list(y = scales_y)) +
  scale_x_continuous(expand = expansion(add = c(50,50)),
                   breaks = seq(-4000,-500,500),
                   minor_breaks = seq(-4000,-500,100)) +
  labs(fill = "") +
  xlab("Age BCE") +
  ylab("") +
  scale_fill_manual(values = taxaColors,
                    breaks = taxalist) +
  theme_bw() +
  theme(legend.position = "bottom",
        panel.grid.major.x = element_line(colour = "grey70"),
        strip.text.y = element_text(size = 11,
                                    face = "bold"),
        axis.text.y = element_text(size = 7,
                                   colour = "black"),
        axis.title.x = element_text(face = "bold"),
        axis.text.x = element_text(colour = "black"),
        legend.key.height = unit(.25,"cm"),
        legend.key.width = unit(.5, "cm"),
        legend.key = element_rect(colour = "black",
                                  size = .5),
        legend.spacing.y = unit(.25, "cm"),
        legend.text = element_text(margin = margin(r = .5, unit = "cm"))) +
  guides(fill = guide_legend(ncol = 4,
                             byrow = T))

p <- layout +
  geom_col(data = subset(archbot_final, var == "RI"), aes(y = score), position = "fill") +
  geom_col(data = subset(archbot_final, var == "proportion"), aes(y = score), position = "fill") +
  geom_col(data = subset(archbot_final, var == "counts"), aes(y = score), position = "stack") +
  geom_col(data = subset(archbot_final, var == "sites"), aes(y = score), position = "stack") +
  geom_col(data = subset(archbot_final, var == "mass finds"), aes(y = score), position = "stack") +
  geom_col(data = subset(archbot_final, var == "features"), aes(y = score), position = "stack")

path <- paste0("../plots/archaeobotanic data mass find and feature counts_",region,"_",maxAgeTimeFrame,"_to_",
               minAgeTimeFrame,"_bins_",tw_step,"_filter_",minCountsFilter,".png")
ggsave(plot = p,
       filename = path,
       dpi = 300,
       width = 15,
       unit = "cm")
rm(path,p,layout)

################## PLOT: DATA WITH MASS FIND PROPORTION ########################

layout <- ggplot(archbot_final %>%
                   dplyr::filter(var %in% c("proportion","RI","counts","rel mass finds","sites")), 
                                 aes(age, fill = factor(taxa, levels = taxalist))) +
  facet_grid_sc(rows = vars(factor(var,
                                   levels = c("proportion","RI","counts","rel mass finds","sites"))),
                scales = list(y = scales_y)) +
  scale_x_continuous(expand = expansion(add = c(50,50)),
                     breaks = seq(-4000,-500,500),
                     minor_breaks = seq(-4000,-500,100)) +
  labs(fill = "") +
  xlab("Age BCE") +
  ylab("") +
  scale_fill_manual(values = taxaColors,
                    breaks = taxalist) +
  theme_bw() +
  theme(legend.position = "bottom",
        panel.grid.major.x = element_line(colour = "grey70"),
        strip.text.y = element_text(size = 11,
                                    face = "bold"),
        axis.text.y = element_text(size = 7,
                                   colour = "black"),
        axis.title.x = element_text(face = "bold"),
        axis.text.x = element_text(colour = "black"),
        legend.key.height = unit(.25,"cm"),
        legend.key.width = unit(.5, "cm"),
        legend.key = element_rect(colour = "black",
                                  size = .5),
        legend.spacing.y = unit(.25, "cm"),
        legend.text = element_text(margin = margin(r = .5, unit = "cm"))) +
  guides(fill = guide_legend(ncol = 4,
                             byrow = T))

p <- layout + 
  geom_col(data = subset(archbot_final, var == "RI"), aes(y = score), position = "fill") +
  geom_col(data = subset(archbot_final, var == "proportion"), aes(y = score), position = "fill") +
  geom_col(data = subset(archbot_final, var == "counts"), aes(y = score), position = "stack") +
  geom_col(data = subset(archbot_final, var == "sites"), aes(y = score), position = "stack") +
  geom_col(data = subset(archbot_final, var == "rel mass finds"), aes(y = score), position = "stack")

path <- paste0("../plots/archaeobotanic data mass find proportion_",region,"_",maxAgeTimeFrame,"_to_",
               minAgeTimeFrame,"_bins_",tw_step,"_filter_",minCountsFilter,".png")
ggsave(plot = p,
       filename = path,
       dpi = 300,
       width = 15,
       unit = "cm")
rm(path,p,layout)

####################### PLOT: CLIMATE COMPARISON ###############################

data_all <- clim_data %>%
  dplyr::rename(score = locfit_200) %>%
  dplyr::bind_rows(archbot_final)

layout <- ggplot(data_all %>%
                   dplyr::filter(var %in% c("prec","temp","proportion","RI")), aes(age, fill = factor(taxa, levels = taxalist))) +
  facet_grid_sc(rows = vars(factor(var,
                                   levels = c("prec","temp","proportion","RI"))),
                scales = list(y = scales_y)) +
  scale_x_continuous(expand = expansion(add = c(50,50)),
                     breaks = seq(-4000,-500,500),
                     minor_breaks = seq(-4000,-500,100)) +
  labs(fill = "") +
  xlab("Age BCE") +
  ylab("") +
  scale_fill_manual(values = taxaColors,
                    breaks = taxalist) +
  theme_bw() +
  theme(legend.position = "bottom",
        panel.grid.major.x = element_line(colour = "grey70"),
        strip.text.y = element_text(size = 11,
                                    face = "bold"),
        axis.text.y = element_text(size = 7,
                                   colour = "black"),
        axis.title.x = element_text(face = "bold"),
        axis.text.x = element_text(colour = "black"),
        legend.key.height = unit(.25,"cm"),
        legend.key.width = unit(.5, "cm"),
        legend.key = element_rect(colour = "black",
                                  size = .5),
        legend.spacing.y = unit(.25, "cm"),
        legend.text = element_text(margin = margin(r = .5, unit = "cm"))) +
  guides(fill = guide_legend(ncol = 4,
                             byrow = T))

p <- layout + 
  geom_col(data = subset(data_all, var %in% c("RI","proportion")), aes(y = score), position = "fill") +
  geom_ribbon(data = subset(data_all, var == "prec"), aes(ymax = upper, ymin = lower), fill = "grey90") +
  geom_line(data = subset(data_all, var == "prec"), aes(y = score), size = .5) +
  geom_ribbon(data = subset(data_all, var == "temp"), aes(ymax = upper, ymin = lower), fill = "grey90") +
  geom_line(data = subset(data_all, var == "temp"), aes(y = score), size = .5) +
  geom_text(data = subset(data_all, var == "prec"), aes(label = paste0("n=",prec_label), y=prec_label_y, x=-500), size = 2.5) +
  geom_text(data = subset(data_all, var == "temp"), aes(label = paste0("n=",temp_label), y=temp_label_y, x=-500), size = 2.5)

path <- paste0("../plots/comparison_climate_archaeobotany_",region,"_",maxAgeTimeFrame,"_to_",
               minAgeTimeFrame,"_bins_",tw_step,"_filter_",minCountsFilter,"_",text1,".png")
ggsave(plot = p,
       filename = path,
       dpi = 300,
       width = 15,
       unit = "cm")
rm(path,p,layout,text1)